/*
 * Copyright (C) 2020 Daniel Nicoletti <dantti12@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "async.h"
#include "context.h"

#include <QPointer>
#include <QLoggingCategory>

Q_LOGGING_CATEGORY(CUTELYST_ASYNC, "cutelyst.async", QtInfoMsg)

using namespace Cutelyst;

namespace Cutelyst {

class ASyncPrivate {
public:
    ASyncPrivate(Context *_c) : c(_c) {}
    ~ASyncPrivate() {
        if (!c.isNull()) {
            qDebug(CUTELYST_ASYNC) << "Attaching async" << c;
            c->attachAsync();
        }
    }

    QPointer<Context> c;
};

}

ASync::ASync()
{

}

ASync::ASync(Context *c)
{
    qDebug(CUTELYST_ASYNC) << "Detaching async" << c;
    c->detachAsync();
    d = QSharedPointer<ASyncPrivate>(new ASyncPrivate(c));
}

ASync::ASync(const ASync &other)
{
    d = other.d;
}

ASync::~ASync()
{

}

ASync &ASync::operator =(const ASync &copy)
{
    d = copy.d;
    return *this;
}
